/*************************************************************************************************************
   Encontra contornos em uma imagem.

**************************************************************************************************************/

#include <opencv\cv.h>
#include <opencv\highgui.h>

IplImage*      g_image    = NULL;
IplImage*      g_gray    = NULL;
int            g_thresh  = 100;
CvMemStorage*  g_storage  = NULL;

void on_trackbar(int)
{
	if( g_storage==NULL )
	{
	    //http://opencv.willowgarage.com/documentation/operations_on_arrays.html#createimage
		g_gray = cvCreateImage( cvGetSize(g_image), 8, 1 ); // Tamanho. número de bits por canal, número de canais
        //http://opencv.willowgarage.com/documentation/dynamic_structures.html#cvCreateMemStorage
		g_storage = cvCreateMemStorage(0);
    }
	else {
		cvClearMemStorage( g_storage );
	}

    //http://opencv.willowgarage.com/documentation/miscellaneous_image_transformations.html#cvCvtColor
	cvCvtColor( g_image, g_gray, CV_BGR2GRAY );

    //http://opencv.willowgarage.com/documentation/miscellaneous_image_transformations.html#cvThreshold
	cvThreshold( g_gray, g_gray, g_thresh, 255, CV_THRESH_BINARY );

    CvSeq* contours = 0;

    //http://opencv.willowgarage.com/documentation/structural_analysis_and_shape_descriptors.html#cvFindContours
    cvFindContours( g_gray, g_storage, &contours );

    // Limpa a imagem
	cvZero( g_gray );

	// Desenha os contornos, se houver
	//http://opencv.willowgarage.com/documentation/drawing_functions.html#cvDrawContours
	if( contours )
		cvDrawContours( g_gray, contours, cvScalarAll(255), cvScalarAll(255), 100 );
    // exibe a imagem
	cvShowImage( "Contours", g_gray );
}

int main( int argc, char** argv )
{

    // Documentacao: http://opencv.willowgarage.com/documentation
    // Página de Busca de Funções:   http://opencv.willowgarage.com/documentation/search.html

    //http://opencv.willowgarage.com/documentation/reading_and_writing_images_and_video.html#cvLoadImage
	IplImage* image = cvLoadImage("abstrato.jpg", 1);
	g_image = cvLoadImage("abstrato.jpg", 1);

    // http://opencv.willowgarage.com/documentation/user_interface.html#cvNamedWindow
	cvNamedWindow( "Contours", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Original", CV_WINDOW_AUTOSIZE );

    //http://opencv.willowgarage.com/documentation/user_interface.html#cvShowImage
	cvShowImage( "Original", image );

	//http://www.cognotics.com/opencv/docs/1.0/ref/opencvref_highgui.htm#decl_cvCreateTrackbar
	cvCreateTrackbar( "Limiar", "Contours", &g_thresh, 255, on_trackbar );

	on_trackbar(0);
	cvWaitKey();
	return 0;
}
